/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

%{

// includes

#include <uds/stl_util.hh>
#include <uds/str_util.hh>
#include <string>
#include "class.hh"
#include "clfilt.hh"
#include "hierarchy.hh"
#include "table.hh"
#include "vtable.hh"

using namespace std;
using namespace uds;


#if 0
#define LOG_OUT clog<<nl<<'\t'<<current_input_pos()<<'\t'<<__LINE__<<'\t'\
	<<( ( yy_start - 1 ) / 2 )<<'\t'<<string( yytext, yyleng )<<endl
#else
#define LOG_OUT
#endif

%}

%option c++
%option noyywrap
%option prefix="ClFilt_base"
%option yyclass="ClFilt"

%x tables
%x number
%x vtable
%x vtable_data

CLNAME	([[:alnum:]_:]+("<"[[:alnum:]_:,*& <>()]+">")?)+
HEX		[0-9a-fA-F]
VTABLE	"_ZTV"[^ \t\n:]+

%%

"Class ".*(("<anonymous "("struct"|"union")">")|"<unnamed>")\n? {
	LOG_OUT;
	while ( yyinput() != '\n' )
		skip();
	}

"Class "{CLNAME} {
	LOG_OUT;
	string name( yytext + 6, yyleng - 6 );
	if ( name != cname )
	{
		cname = name;
		cl = &hier.AddClass( cname );
	}
	}

"size="[[:digit:]]+ {
	LOG_OUT;
	size_t csize = VarConv< size_t >( string( yytext + 5, yyleng - 5 ) );
	cl->SetSize( csize );
	BEGIN tables;
	skip();
	}

"Vtable for "{CLNAME} {
	LOG_OUT;
	cname.assign( yytext + 11, yyleng - 11 );
	cl = &hier.AddClass( cname );
	BEGIN vtable;
	}



<vtable>{VTABLE}": " {
	LOG_OUT;
	tname.assign( yytext, yyleng - 2 );
	}

<vtable>[[:digit:]]+" entries" {
	LOG_OUT;
	size_t ne = VarConv< size_t >( string( yytext, yyleng - 8 ) );
	if ( vtbl )
		_uds_abort;
	vtbl = &hier.AddVTable( tname, *cl, ne );
	BEGIN vtable_data;
	}

<vtable_data>^"\n" {
	LOG_OUT;
	vtbl = 0;
	BEGIN INITIAL;
	}

<vtable_data>^[[:digit:]]+ {
	LOG_OUT;
	cnum = VarConv< int >( string( yytext, yyleng ) );
	if ( cnum % sizeof( void* ) )
		_uds_abort;
	}

<vtable_data>[[:alpha:]_].*"\n" {
	LOG_OUT;
	string sym( yytext, yyleng );
	if ( sym.find( "_ZTI" ) != string::npos )
	{
		string fname = tname + "_" + StringVar( ftab_q.size() );
		fname.at( 3 ) = 'I';

		vtbl->Assign( cnum / sizeof( void* ), SYM_PREFIX + fname );
		Table& t = cl->AddFTable( fname, hier.TableGroup(), cnum );
		ftab_q.push( &t );
	}
	}



<tables>" "*{CLNAME}" (".*"virtual non-canonical" {
	LOG_OUT;
	}

<tables>" "*{CLNAME}" (" {
	LOG_OUT;
	size_t ns = 0;
	for ( ; yytext[ns] == ' '; ++ns );
	if ( ns % 2 )
		_uds_abort;
	cname.assign( yytext + ns, yyleng - ns - 2 );
	skip( ')' );
	parse( number );
	cl->SetTableOffset( hier.GetClass( cname ), cnum, ns / 2 );
	}

<tables>"vptr=((".*") + " {
	LOG_OUT;
	parse( number );
	if ( ftab_q.empty() )
		_uds_abort;
	if ( vbase )
	{
		vbase = false;
		vbfo_v.push_back( make_pair( cname, cnum ) );
	}
	else
		proc_ftable( cname, cnum );
	}

<tables>"primary-for "{CLNAME} {
	LOG_OUT;
	string pname( yytext + 12, yyleng - 12 );
	if ( pname == cl->Name() )
		cl->MainFTable().SetParent( hier.GetClass( cname ).MainFTable() );
	skip( ')' );
	}

<tables>"virtual canonical" {
	LOG_OUT;
	vbase = true;
	cl->AddVBase( hier.GetClass( cname ) );
	}

<tables>^"\n" {
	LOG_OUT;
	cl = 0;
	for ( size_t i = 0; i < vbfo_v.size(); ++i )
		proc_ftable( vbfo_v[i].first, vbfo_v[i].second );
	vbfo_v.clear();
	if ( !ftab_q.empty() )
		_uds_abort;
	BEGIN INITIAL;
	}



<number>"-"?[[:digit:]]+ {
	LOG_OUT;
	cnum = VarConv< int >( string( yytext, yyleng ) );
	return 0;
	}



<*>.|\n



<<EOF>> {
	LOG_OUT;
	if ( YY_START != INITIAL )
		_uds_abort;
	return 0;
	}

%%

ClFilt::ClFilt( istream* in, Hierarchy& h ) :
	yyFlexLexer( in, 0 ), nl( 0 ), hier( h ), cl( 0 ), vtbl( 0 ),
	vbase( false )
{
	yy_flex_debug = 1;
}

ClFilt::~ClFilt()
{
}

void
ClFilt::proc_ftable( const string& cname, int cnum )
{
	Table& t = *ftab_q.front();
	ftab_q.pop();
	t.AdjustFrostVTableOffset( -cnum );
	t.SetParent( hier.GetClass( cname ).MainFTable() );
}

void
ClFilt::skip( char x )
{
	while ( true )
	{
		char c = yyinput();
		
		if ( c == x )
			break;
		else if ( c == EOF )
			_uds_abort;
	}
}

int
ClFilt::current_input_pos() const
{
	return static_cast< int >( yyin->tellg() ) - yy_n_chars
		+ static_cast< int >( yy_c_buf_p - yy_current_buffer->yy_ch_buf );
}

void
ClFilt::begin( int sc )
{
	BEGIN sc;
}

void
ClFilt::parse( int sc )
{
	int old_sc = YY_START;
	BEGIN sc;
	++nl;
	
	yylex();

	--nl;
	BEGIN old_sc;
}
