/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_dbase_hh
#define _frost_dbase_hh


// includes

#include <iostream>
#include <string>
#include <utility>
#include <vector>

using namespace std;


/** dispatcher base class
	\author Julian v. Bock */
class DispBase
{
public:
	/** \param n dispatcher name */
	DispBase( const string& n );
	
	/** destructor */
	virtual
	~DispBase();

	/** \return dispatcher name */
	virtual const string&
	Name() const { return name; }

	/** \return true if the dispatcher thunk would do nothing */
	virtual bool
	Empty() const { return tp_adj.empty(); }

	/** adjusts the 'this' pointer of an argument
		\param s stack offset
		\param d 'this' pointer delta */
	virtual void
	Adjust( int s, int d );

	/** prints the dispatcher assembly code
		\param out output stream
		\return output stream */
	virtual ostream&
	EmitCode( ostream& out ) const = 0;

	/** \return if the dispatcher is global */
	virtual bool
	Global() const = 0;

protected:
	typedef pair< int, int > so_pair;
	typedef vector< so_pair > so_vec;

	/** \return vector to 'this' pointer adjustments */
	const so_vec&
	TP_Adj() const { return tp_adj; }
	
private:
	/// 'this' pointer adjustments
	so_vec	tp_adj;
	/// dispatcher thunk name
	string	name;
};

#endif // _frost_dbase_hh
