/*
 * Frost - use C++ multi methods as if they were a native feature
 * Copyright (C) 2000, 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include "ascode.hh"
#include "disp.hh"
#include "table.hh"
#include "tablegrp.hh"


// Dispatcher implementation

Dispatcher::Dispatcher( const string& n, TableGrp& tg, int s, int l, int c,
						int o, int f, bool g ) :
	DispBase( n ), tgrp( tg ), section( s ), level( l ), c_offs( c ),
	st_offs( o ), ft_offs( f ), global( g )
{
}

Dispatcher::~Dispatcher()
{
}

ostream&
Dispatcher::EmitCode( ostream& out ) const
{
	out<<endl;
	
	DefineFunction( out, Name(), global );

	for ( size_t i = TP_Adj().size(); i > 0; )
	{
		--i;
		
		size_t so = TP_Adj()[i].first;
		size_t dt = TP_Adj()[i].second;
		
		// adjust 'this' pointer
		AdjustThis( out, so, dt, i == 0 );
	}

	if ( Empty() || TP_Adj()[0].first != st_offs )
	{
		// get object address
		StackDeref( out, st_offs );
	}
		
	// get virtual table address
	WRegDeref( out, 0 );
		
	// get frost table address
	WRegDeref( out, ft_offs );
		
	// jump to next function
	WRegJump( out, c_offs + tgrp.FragmentOffset( section, level ) );
	
	return out;
}
