/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

%{

// includes

#include <uds/stdexcpt.hh>
#include <uds/str_util.hh>
#include "class.hh"
#include "fnfilt.hh"
#include "function.hh"
#include "hierarchy.hh"
#include "progb.hh"
#include "type.hh"

using namespace uds;


class Class;


#if 0
#define LOG_OUT clog<<nl<<'\t'<<current_input_pos()<<'\t'<<__LINE__<<'\t'\
	<<( ( yy_start - 1 ) / 2 )<<'\t'<<string( yytext, yyleng )<<endl
#else
#define LOG_OUT
#endif

%}

%option c++
%option noyywrap
%option prefix="FnFilt_base"
%option yyclass="FnFilt"

%x farg
%x fname
%x getid
%x vainfo

SID	[[:alnum:]_]+
ID	" "*[[:alnum:]_:]+

%%

{SID} {
	LOG_OUT;
	func = new Function( string( yytext, yyleng ) );
	BEGIN fname;
	}



<fname>{SID}"::" {
	LOG_OUT;
	if ( aname.size() )
		aname += "::";
	aname.append( yytext, yyleng - 2 );
	}

<fname>((.+"::")?"operator"[^[:alnum:]_:])|[~(\n]|"thunk "|"guard "|"vtable "|"VTT "|"typeinfo " {
	LOG_OUT;
	finish_func( *yytext != '\n', true );
	}

<fname>{SID}"(" {
	LOG_OUT;
	string sym( yytext, yyleng - 1 );
	sfunc = false;
	const size_t sps = strlen( SYM_PREFIX );
	
	if ( sym.substr( 0, sps ) != SYM_PREFIX )
		finish_func( true, true );
	else
	{
		yyless( sps );
		if ( sym[sps] == 'p' )
			sym.erase( sps, 1 );
		parse( vainfo );
		skip( '(' );

		func->SetFName( sym );
		asize = 0;
		proc_arg();
		BEGIN farg;
	}
	}



<farg>"(*)" {
	LOG_OUT;
	aname.append( yytext, yyleng );
	asize = sizeof( void ( * )() );
	}

<farg>{ID}\n {
	LOG_OUT;
	finish_func( false, true );
	}

<farg>{ID} {
	LOG_OUT;
	aname.append( yytext, yyleng );
	}

<farg>(" "?"volatile")?(" "?"const")?[*&]?(", "|(")"[ \n])) {
	LOG_OUT;
	if ( aname.size() )
	{
		if ( yyleng >= 3 )
		{
			char c = yytext[yyleng - 3];
			if ( c == '*' || c == '&' )
				asize = sizeof( void* );
		}
		
		proc_arg( string( yytext, yyleng - 2 ) );
	}

	if ( yytext[yyleng - 2] == ')' )
	{
		// assume const method if yytext[yyleng - 1] == ' '

		bool cm = yytext[yyleng - 1] != '\n';
		if ( cm )
			func->SetConst();
		
		finish_func( cm, false );
	}
	}
	


<getid,farg,fname>[(<] {
	LOG_OUT;
	aname += *yytext;
	parse( getid );
	}

<getid>"::*)(" {
	LOG_OUT;
	aname.append( yytext, yyleng );
	if ( !tmplarg )
		asize = sizeof( void ( FnFilt::* )() );
	}

<getid>[)>] {
	LOG_OUT;
	aname += *yytext;
	return 0;
	}

<getid>.|\n {
	LOG_OUT;
	aname += *yytext;
	}



<vainfo>"s" {
	LOG_OUT;
	sfunc = true;
	}

<vainfo>"p" {
	LOG_OUT;
	func->SetPure();
	}

<vainfo>"v"[[:digit:]]+ {
	LOG_OUT;
	va_queue.push( VarConv< size_t >( string( yytext + 1, yyleng - 1 ) ));
	}

<vainfo>"_" {
	LOG_OUT;
	return 0;
	}

<vainfo>. {
	LOG_OUT;
	_uds_abort;
	}



<*>.|\n {
	LOG_OUT;
	}



<<EOF>> {
	LOG_OUT;
	if ( YY_START != INITIAL )
		_uds_abort;
	return 0;
	}

%%

FnFilt::FnFilt( istream* in, Hierarchy& h ) :
	yyFlexLexer( in, 0 ), nl( 0 ), hier( h ), func( 0 ), asize( -1 ),
	tmplarg( false ), sfunc( false )
{
	yy_flex_debug = 1;
}

FnFilt::~FnFilt()
{
}

void
FnFilt::proc_arg( const string& ea )
{
	bool va = false;
	if ( va_queue.size() && va_queue.front() == func->ArgCount() )
	{
		va = true;
		va_queue.pop();
	}

	Class* cl = hier.QueryClass( aname );

	if ( cl && func->ArgCount() == 0 )
		asize = sfunc ? 0 : sizeof( void* );

	func->AddArg( Type( hier, aname, ea, asize, va ) );
	
	aname.clear();
	asize = -1;
}

void
FnFilt::finish_func( bool s, bool d )
{
	if ( va_queue.size() )
		_uds_abort;
	
	if ( s )
		skip();

	if ( d )
		_frost_Log( LOG_DEBUG + 2, "dropped (not a function)" );
	else
		hier.AddFunc( func );

	aname.clear();
	func = 0;
	asize = -1;
	
	BEGIN INITIAL;
}

void
FnFilt::skip( char x )
{
	while ( true )
	{
		char c = yyinput();
		
		if ( c == x )
			break;
		else if ( c == EOF )
			_uds_abort;
	}
}

int
FnFilt::current_input_pos() const
{
	return static_cast< int >( yyin->tellg() ) - yy_n_chars
		+ static_cast< int >( yy_c_buf_p - yy_current_buffer->yy_ch_buf );
}

void
FnFilt::begin( int sc )
{
	BEGIN sc;
}

void
FnFilt::parse( int sc )
{
	int old_sc = YY_START;
	BEGIN sc;
	++nl;
	
	yylex();

	--nl;
	BEGIN old_sc;
}
