/*
 * Frost - use C++ multi methods as if they were a native feature
 * Copyright (C) 1999, 2000, 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_fpp_hh
#define _frost_fpp_hh


// includes

#include <string>

#ifndef yyFlexLexer
#define _CPP_BACKWARD_IOSTREAM_H 1
#include "FlexLexer.h"
#endif

using namespace std;


/** The Frost preprocessor class. It makes the input source code suitable
	for a C++ compiler
	\author Julian v. Bock
	\see FrostLexer */
class FrostPP : public yyFlexLexer
{
public:
	/** \param in input stream
		\param out output stream
		\param file primary file name (needed for debugging output) */
	FrostPP( istream* in, ostream* out, const string& file );
	
	/** destructor */
	virtual
	~FrostPP();
	
	/** flex lexer method */
	virtual int
	yylex();
	
private:
	/// yylex nesting level (used for debugging)
	int		nl;
	/// current multi method declaration
	string	decl;
	/// varg info string
	string	vargs;
	/// varg string offset
	size_t	va_offs;
	/// current varg
	size_t	va_cur;
	/// skip char
	char	skipc;
	/// 'print unmodified decl' flag
	bool	pdecl;
	/// 'inline function' flag
	bool	inlf;

	/** \return current input position */
	int
	current_input_pos() const;

	/** sets a new start condition
		\param sc new start condition */
	void
	begin( int sc );

	/** calls yylex() with a new state. the old state is preserved.
		note that this function is not exception safe.
		\param sc new start condition */
	void
	parse( int sc );

	/** skips string / character constants
		\param app 'append to decl' flag */
	void
	skip_sc( bool app );
};

#endif // _frost_fpp_hh
