/*
 * Frost - use C++ multi methods as if they were a native feature
 * Copyright (C) 1999, 2000, 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

%{

// includes

#include <uds/action.hh>
#include <uds/str_util.hh>
#include <sstream>
#include <string>
#include "config.h"
#include "exception.hh"
#include "fpp.hh"
#include "progb.hh"
#include "config.h"

using namespace std;


#if 0
#define LOG_OUT clog<<nl<<'\t'<<current_input_pos()<<'\t'<<__LINE__<<'\t'\
	<<( ( yy_start - 1 ) / 2 )<<'\t'<<string( yytext, yyleng )<<endl
#else
#define LOG_OUT
#endif

%}

%option c++
%option noyywrap
%option prefix="FrostPP_base"
%option yyclass="FrostPP"

%x mm_head
%x mm_tail
%x skip_br

SID		[[:alpha:]_][[:alnum:]_]*
MM		([;{}]|("#".*\n))[^;{}#'"]*"("[^;{]*"virtual"[ \t\n]

%%

{MM} {
	/** ' */
	LOG_OUT;
	if ( Log( LOG_DEBUG + 1 ) )
	{
		string lm( yytext, yyleng );
		for ( size_t i = 0; i < lm.size(); ++i )
			if ( lm[i] == '\n' )
				lm[i] = ' ';

		Log( LOG_DEBUG + 1, "mm detected: " + lm );
	}

	bool yl = false;
	
	if ( *yytext == '#' )
	{
		int n = 1;
		while ( yytext[n++] != '\n' );

		*yyout<<string( yytext, n );
		yyless( n );
	}
	else
	{
		yl = *yytext == '{';
		
		*yyout<<*yytext;
		yyless( 1 );
	}
	va_cur = 0;
	vargs = SYM_PREFIX;
	inlf = false;
	BEGIN mm_head;

	if ( yl )
		yylex();
	}

"{" {
	LOG_OUT;
	ECHO;
	yylex();
	}

"}" {
	LOG_OUT;
	ECHO;
	return 0;
	}



<mm_head>("public"|"protected"|"private")[ \t\n]*":" {
	LOG_OUT;
	ECHO;
	}

<mm_head,mm_tail>"static" {
	LOG_OUT;
	_frost_Log( LOG_DEBUG + 1, "static" );
	decl.append( yytext, yyleng );
	vargs += "s";
	}

<mm_head,mm_tail>"virtual" {
	LOG_OUT;
	_frost_Log( LOG_DEBUG + 1, "virtual arg " + StringVar( va_cur ) );
	vargs += "v";
	vargs += StringVar( va_cur );
	}

<mm_head>"inline" {
	LOG_OUT;
	_frost_Log( LOG_DEBUG + 1, "inline" );
	decl.append( yytext, yyleng );
	inlf = true;
	}

<mm_head,mm_tail>{SID} {
	LOG_OUT;
	decl.append( yytext, yyleng );
	}

<mm_head>("::"[ \t\n]*)?{SID}[ \t\n]*"(" {
	LOG_OUT;
	va_offs = decl.size();
	if ( *yytext == ':' )
	{
		pdecl = false;
		for ( int i = 2; ; ++i )
		{
			if ( i == yyleng )
				_uds_abort;

			char c = yytext[i];
			if ( c != ' ' && c != '\t' && c != '\n' )
			{
				va_offs += i;
				break;
			}
		}
	}
	else
		pdecl = true;
	
	va_cur = 1;
	decl.append( yytext, yyleng );
	BEGIN mm_tail;
	}

<mm_head>[;{] {
	LOG_OUT;
	_uds_abort;
	}

<mm_tail>[<(] {
	LOG_OUT;
	decl += *yytext;
	skipc = *yytext;
	parse( skip_br );
	}

<mm_tail>, {
	LOG_OUT;
	++va_cur;
	decl += *yytext;
	}

<mm_tail>"="[ \t\n]*"0"[ \t\n]*";" {
	LOG_OUT;
	_frost_Log( LOG_DEBUG + 1, "pure" );
	if ( !inlf )
	{
		decl.insert( 0, "inline " );
		va_offs += 7;
	}
	const char* noret = "{ " SYM_PREFIX "noret(); }";
	for ( int i = strlen( noret ) - 1; i >= 0; --i )
		unput( noret[i] );
	vargs.insert( strlen( SYM_PREFIX ), "p" );
	}

<mm_tail>[;{] {
	LOG_OUT;
	yyless( 0 );
	if ( pdecl )
	{
		for ( size_t i = 0; i < decl.size(); ++i )
		{
			*yyout<<decl[i];
			
			if ( decl[i] == '\n' )
				decl[i] = ' ';
		}
	}
	*yyout<<"; ";
	decl.insert( va_offs, vargs + "_" );
	*yyout<<decl;
	decl.erase();
	BEGIN INITIAL;
	}

<mm_head,mm_tail>['"] {
	/* ' { */
	LOG_OUT;
	skip_sc( true );
	}



<skip_br>[<(] {
	LOG_OUT;
	decl += *yytext;
	VRemember< char > vr( skipc, *yytext );
	parse( skip_br );
	}

<skip_br>[>)] {
	LOG_OUT;
	decl += *yytext;
	return 0;
	}



<mm_head,mm_tail,skip_br>.|\n {
	LOG_OUT;
	decl += *yytext;
	}



<*>['"] {
	/* ' { */
	LOG_OUT;
	skip_sc( false );
	}



.|\n {
	LOG_OUT;
	ECHO;
	}



<<EOF>> {
	LOG_OUT;
	if ( YY_START != INITIAL )
		throw ParserError( "unexpected end of file" );
	return 0;
	}

%%

FrostPP::FrostPP( istream* in, ostream* out, const string& file ) :
	yyFlexLexer( in, out ), nl( 0 )
{
	yy_flex_debug = 1;

	// print the frost noreturn dummy function declaration

	*out<<"# 1 \""<<file<<"\"\n"
		"extern \"C\" { extern void " SYM_PREFIX "noret() "
		"__attribute__(( noreturn )); }\n";
}

FrostPP::~FrostPP()
{
}

int
FrostPP::current_input_pos() const
{
	return static_cast< int >( yyin->tellg() ) - yy_n_chars
		+ static_cast< int >( yy_c_buf_p - yy_current_buffer->yy_ch_buf );
}

void
FrostPP::begin( int sc )
{
	BEGIN sc;
}

void
FrostPP::parse( int sc )
{
	int old_sc = YY_START;
	BEGIN sc;
	++nl;
	
	yylex();

	--nl;
	BEGIN old_sc;
}

void
FrostPP::skip_sc( bool app )
{
	int esc = 2;
	for ( char c = *yytext, delim = c; ; c = yyinput() )
	{
		if ( app )
			decl += c;
		*yyout<<c;

		if ( esc )
			--esc;
		
		if ( c == EOF )
			_uds_abort;
		else if ( c == '\\' && !esc )
			esc = 2;
		else if ( c == delim && !esc )
			break;
	}
}
