/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_mm_hh
#define _frost_mm_hh


// includes

#include <set>
#include <vector>
#include "dbase.hh"

using namespace std;


class Class;
class Function;
class Table;
class TableGrp;


/** multi method tree class
	\author Julian v. Bock */
class MMethod
{
public:
	typedef vector< DispBase* > disp_vec;
	typedef const Function* m_ptr;
	
	/** \param f first function that is part of the multi method tree
		\param tg frost table group */
	explicit
	MMethod( const Function& f, TableGrp& tg );

	/** destructor */
	~MMethod();

	/** \return demangled name of the root function */
	string
	Name() const;

	/** adds a function to the mm tree if it is compatible
		\param f new function
		\return true if the function was added */
	bool
	AddMemberMaybe( const Function& f );

	/** builds frost table fragments and dispatcher functions
		\param disps vector of dispatcher functions. dispatchers for this
		multi method are added to this vector
		\return true if the multi method was built successfully */
	bool
	BuildMM( disp_vec& disps ) const;

	/** \return fwva root function */
	m_ptr
	Root() const { return root; }

private:
	typedef set< m_ptr > mf_set_t;
	typedef set< Table* > tab_set;
	typedef tab_set::iterator ts_iter;
	typedef vector< Table* > tab_vec;
	typedef vector< tab_set > ts_vec;
	typedef vector< size_t > so_vec;
	typedef vector< Class* > cl_vec;

	/** builds frost table fragments and dispatcher functions
		\param disps vector of dispatcher functions. dispatchers for this
		multi method are added to this vector
		\param fixt fixed tables
		\param tree table sets
		\param soffs virtual argument stack offset vector
		\param rcls root classes
		\param dnt encoded dispatcher name argument list
		\param lev dispatcher level
		\param succ 'success' flag
		\return true if the multi method was built successfully */
	bool
	build_mm( disp_vec& disps, const tab_vec& fixt, const ts_vec& tree,
			  const so_vec& soffs, const cl_vec& rcls, const string& dnt,
			  int lev, bool succ ) const;

	/** resolves a multi method function call
		\param tv frost table vector
		\param err error message (is filled on error)
		\return address of function that is called or 0 if an error
		occurred */
	m_ptr
	resolve_call( const tab_vec& tv, string& err ) const;

	/// mm tree member functions
	mf_set_t	mf_set;
	/// root function
	m_ptr		root;
	/// frost table group
	TableGrp&	tgrp;
	/// unique id
	int			uid;
};

#endif // _frost_mm_hh
