/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <signal.h>
#include <uds/btrace.hh>
#include <uds/exit.hh>
#include <uds/str_util.hh>
#include "prog.hh"
#include "sighandlers.hh"


// no comment on this one...
typedef struct sigaction sigaction_t;


// ProgramState implementation

void
ProgramState::SetSigHandlers() const
{
	log( LOG_DEBUG, "installing signal handlers" );

	sigaction_t sa;
	sa.sa_handler = &fatal_handler;
	sa.sa_flags = SA_ONESHOT;
	_uds_CkStdCall( sigemptyset( &sa.sa_mask ) );
	_uds_CkStdCall( sigaction( SIGABRT, &sa, 0 ) );
	_uds_CkStdCall( sigaction( SIGSEGV, &sa, 0 ) );
	_uds_CkStdCall( sigaction( SIGFPE, &sa, 0 ) );
}


// signal handlers

void
fatal_handler( int sig )
{
	// going down

	sigaction_t sa;
	sa.sa_handler = SIG_DFL;
	sa.sa_flags = 0;
	sigemptyset( &sa.sa_mask );
	sigaction( SIGABRT, &sa, 0 );
	sigaction( SIGSEGV, &sa, 0 );
	sigaction( SIGFPE, &sa, 0 );
	
	stringstream s;
	s<<"caught fatal signal "<<sig<<" -- going down\n"<<Backtrace();
	cerr<<s.str();
	Log( LOG_CRIT, s.str() );

	// bye
	raise( sig );
}
