/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/stdexcpt.hh>
#include <uds/stl_util.hh>
#include "table.hh"
#include "tablegrp.hh"

using namespace uds;


// TableGrp implementation

TableGrp::TableGrp()
{
}

TableGrp::~TableGrp()
{
}

void
TableGrp::AddTrunk( Table& t )
{
	if ( !trunks.insert( &t ).second )
		_uds_abort;
}

void
TableGrp::RemoveTrunk( Table& t )
{
	trunks.erase( &t );
}

int
TableGrp::FragmentOffset( int s, int l ) const
{
	return MapFindC< internal_error >( offsets, make_pair( s, l ) );
}

void
TableGrp::ArrangeFrags()
{
	for ( tbl_set::const_iterator i = trunks.begin(); i != trunks.end(); ++i )
		( *i )->Arrange( 0 );
}

void
TableGrp::EmitCode( ostream& out ) const
{
	for ( tbl_set::const_iterator i = trunks.begin(); i != trunks.end(); ++i )
		( *i )->EmitCode( out );
}

void
TableGrp::FWVAClasses( set< Class* >& cs ) const
{
	cs.clear();
	
	for ( tbl_set::const_iterator i = trunks.begin(); i != trunks.end(); ++i )
		( *i )->InsertOc( cs );
}

void
TableGrp::DumpHierarchies( ostream& out ) const
{
	out<<"(ft-hier";

	for ( tbl_set::const_iterator i = trunks.begin(); i != trunks.end(); ++i )
	{
		out<<' ';
		( *i )->DumpHierarchy( out );
	}

	out<<')';
}

bool
TableGrp::HasFragOffset( const frag_key& fk ) const
{
	return offsets.find( fk ) == offsets.end();
}

void
TableGrp::SetFragOffset( const frag_key& fk, size_t o )
{
	offsets[fk] = o;
}
