/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_tablegrp_hh
#define _frost_tablegrp_hh


// includes

#include <iostream>
#include <map>
#include <set>

using namespace std;


class Table;


/** Table Group class.
	\author Julian v. Bock */
class TableGrp
{
public:
	typedef set< Table* > tbl_set;
	typedef pair< int, int > frag_key;

	/** default constructor */
	TableGrp();

	/** destructor */
	~TableGrp();

	/** adds a new trunk table.
		\param t new trunk */
	void
	AddTrunk( Table& t );

	/** removes a trunk table.
		\param t new trunk */
	void
	RemoveTrunk( Table& t );

	/** returns the offset of a section / level pair
		\param s section
		\param l level
		\return offset */
	int
	FragmentOffset( int s, int l ) const;

	/** arranges all frost table fragments */
	void
	ArrangeFrags();

	/** prints frost tables to an output stream
		\param out output stream */
	void
	EmitCode( ostream& out ) const;

	/** fills a set with all classes which are involved in fwvas.
		\param cs class set */
	void
	FWVAClasses( set< Class* >& cs ) const;

	/** dumps all table hierarchies to an output stream
		\param out output stream */
	void
	DumpHierarchies( ostream& out ) const;

	/** checks whether the offset for a frag_key has already been set.
		\param fk fragment key
		\return true if the offset has already been set */
	bool
	HasFragOffset( const frag_key& fk ) const;

	/** sets a new fragment key offset.
		\param fk fragment key
		\param o new offset */
	void
	SetFragOffset( const frag_key& fk, size_t o );

private:
	typedef map< frag_key, int > frag_offs;

	/// tables which have no parents
	tbl_set		trunks;
	/// fragment offsets
	frag_offs	offsets;
};

#endif // _frost_tablegrp_hh
