/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/str_util.hh>
#include "ascode.hh"
#include "progb.hh"
#include "titable.hh"
#include "vtable.hh"

using namespace uds;


// VTable implementation

TITable::TITable( const string& n, VTable& vt ) :
	vti_base( n, 0 )
{
	_frost_Log( LOG_DEBUG + 2, "new typeinfo table " + n );

	vt.SetTInfo( *this );
}

TITable::~TITable()
{
}

void
TITable::Assign( size_t s, const string& v )
{
	_frost_Log( LOG_DEBUG + 2, "assigning slot " + StringVar( s )
				+ " of typeinfo table " + Name() + " to " + v );
	
	if ( s >= slots.size() || v.empty() || !slots[s].empty() )
		_uds_abort;

	slots[s] = v;
}

void
TITable::Fill( int v )
{
	_frost_Log( LOG_DEBUG + 2, "filling slot " + StringVar( deltas.size() )
				+ " of typeinfo table " + Name() + " with " + StringVar( v ) );

	deltas.push_back( v );
	slots.push_back( string() );
}

void
TITable::EmitCode( ostream& out ) const
{
	if ( !Complete() )
		throw error( "unable to print incomplete typeinfo table " + Name() );

	vti_base::EmitCode( out );
}
