/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/str_util.hh>
#include "ascode.hh"
#include "class.hh"
#include "progb.hh"
#include "titable.hh"
#include "vtable.hh"

using namespace uds;


// VTable implementation

VTable::VTable( const string& n, Class& c, size_t sz ) :
	vti_base( n, sz ), cl( c ), tit( 0 )
{
	_frost_Log( LOG_DEBUG + 2, "new virtual table " + n + "; "
				+ StringVar( sz ) + " entries" );
	
	if ( sz == 0 )
		_uds_abort;

	cl.SetVtbl( *this );
}

VTable::~VTable()
{
	delete tit;
}

TITable&
VTable::TInfo() const
{
	if ( !tit )
		_uds_abort;

	return *tit;
}

void
VTable::SetTInfo( TITable& ti )
{
	if ( tit )
		_uds_abort;

	tit = &ti;
}

void
VTable::Assign( size_t s, const string& v )
{
	_frost_Log( LOG_DEBUG + 2, "assigning slot " + StringVar( s )
				+ " of virtual table " + Name() + " to " + v );

	// ignore typeinfo pointers. they get assigned when the class file is
	// parsed.

	if ( v.substr( 0, 4 ) == "_ZTI" )
	{
		_frost_Log( LOG_DEBUG + 2, "(typeinfo pointer ignored)" );
		return;
	}
	
	if ( s >= slots.size() || v.empty() || !slots[s].empty() )
		_uds_abort;

	slots[s] = v;
}

void
VTable::Fill( int v )
{
	_frost_Log( LOG_DEBUG + 2, "filling slot " + StringVar( deltas.size() )
				+ " of virtual table " + Name() + " with " + StringVar( v ) );

	deltas.push_back( v );
}

void
VTable::EmitCode( ostream& out ) const
{
	if ( !Complete() )
		throw error( "unable to print incomplete virtual table " + Name() );

	PrintTIString( out, Name() );
	
	DefineVTable( out, Name(), slots.size() + 1 );
	vti_base::EmitCode( out );
	PrintTableEntry( out, "0" );
}


// misc. implementation

ostream&
operator << ( ostream& out, const VTable& vt )
{
	out<<"(\""<<vt.cl.Name()<<"\" "<<static_cast< const vti_base& >( vt );

	if ( vt.tit )
		out<<' '<<*vt.tit;

	return out<<')';
}
