/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _frost_vtable_hh
#define _frost_vtable_hh


// includes

#include "vtibase.hh"

using namespace std;


class Class;
class TITable;


/** virtual table class
	\author Julian v. Bock */
class VTable : public vti_base
{
public:
	/** \param n encoded virtual table name
		\param cl owner class
		\param sz number of entries in virtual table */
	VTable( const string& n, Class& cl, size_t sz );

	/** destructor */
	virtual
	~VTable();

	/** \return owner class */
	Class&
	Owner() { return cl; }

	/** \return typeinfo table */
	TITable&
	TInfo() const;

	/** sets the typeinfo table
		\param ti typeinfo table */
	void
	SetTInfo( TITable& ti );

	/** \return true if the vtable has a typeinfo table */
	bool
	HasTInfo() const { return tit; }

	/** assigns a value to a vtable slot
		\param s vtable slot
		\param v value */
	virtual void
	Assign( size_t s, const string& v );

	/** adds raw data to the vtable
		\param v value */
	virtual void
	Fill( int v );

	/** print the assembler representation of the virtual table to an output
		stream
		\param out output stream */
	virtual void
	EmitCode( ostream& out ) const;

	/** output stream operator
		\param out output stream
		\param cl class */
	friend ostream&
	operator << ( ostream& out, const VTable& cl );

private:
	/// owner class
	Class&		cl;
	/// typeinfo table
	TITable*	tit;
};

#endif // _frost_vtable_hh
