/*
 * frost - C++ multi methods as if they were a native feature
 * Copyright (C) 2001, 2002  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */


// includes

#include <uds/str_util.hh>
#include "ascode.hh"
#include "vtibase.hh"

using namespace uds;


// VTable implementation

vti_base::vti_base( const string& n, size_t sz ) :
	slots( sz ), name( n ), complete( false )
{
	if ( name.empty() )
		_uds_abort;
}

vti_base::~vti_base()
{
}

void
vti_base::EmitCode( ostream& out ) const
{
	if ( !Complete() )
		_uds_abort;

	for ( size_t i = 0; i < slots.size(); ++i )
	{
		string s = slots[i];
		
		if ( int d = deltas[i] )
		{
			if ( d > 0 && !s.empty() )
				s += '+';

			s += StringVar( d );
		}
		else if ( s.empty() )
			s = "0";

		PrintTableEntry( out, s );
	}
}


// misc. implementation

ostream&
operator << ( ostream& out, const vti_base& vt )
{
	out<<"(\""<<vt.name<<"\" "<<vt.slots.size()<<" (";

	bool f = true;
	for ( size_t i = 0; i < vt.deltas.size(); ++i )
	{
		if ( f )
			f = false;
		else
			out<<' ';

		out<<vt.deltas[i];
	}

	out<<") (";

	f = true;
	for ( size_t i = 0; i < vt.slots.size(); ++i )
	{
		if ( !vt.slots[i].empty() )
		{
			if ( f )
				f = false;
			else
				out<<' ';

			out<<"("<<i<<" \""<<vt.slots[i]<<"\")";
		}
	}

	return out<<"))";
}
